<?php

/**
 * @file
 * Hooks and preprocess functions for the GridStack module.
 */

use Drupal\blazy\Blazy;
use Drupal\gridstack\GridStackDefault;
use Drupal\gridstack\Entity\GridStack;

/**
 * Prepares variables for gridstack.html.twig templates.
 */
function template_preprocess_gridstack(&$variables) {
  $element = $variables['element'];
  foreach (['items', 'optionset', 'postscript', 'settings'] as $key) {
    $variables[$key] = isset($element['#' . $key]) ? $element['#' . $key] : [];
  }

  $variables['settings']  += GridStackDefault::htmlSettings();
  $settings                = &$variables['settings'];
  $attributes              = &$variables['attributes'];
  $settings['id']          = $id = Blazy::getHtmlId('gridstack', $settings['id']);
  $settings['class_by_id'] = strpos($id, 'gridstack-') !== FALSE ? str_replace('gridstack-', 'gridstack--', $id) : '';
  $custom_classes          = empty($attributes['class']) ? [] : $attributes['class'];
  $attributes['class']     = array_merge(['gridstack'], $custom_classes);
  $variables['optionset']  = $variables['optionset'] ?: GridStack::loadWithFallback($settings['optionset']);
  $variables['wrapper']    = 'div';

  if ($settings['root']) {
    $variables['wrapper'] = empty($settings['wrapper']) ? 'div' : $settings['wrapper'];
  }

  // Do not leak container wrapper to its children below.
  unset($settings['wrapper']);

  // Process individual item, basically converting array to render array.
  foreach ($variables['items'] as $delta => $item) {
    // Prevents complication with nested grids.
    if (!isset($item['box'])) {
      continue;
    }

    $item_attributes = isset($item['attributes']) ? $item['attributes'] : [];
    $item_settings   = isset($item['settings']) ? array_merge($settings, array_filter($item['settings'])) : $settings;

    // Prevents extracted item elements from being rendered.
    unset($item['attributes'], $item['settings']);

    $box = [
      '#theme'      => 'gridstack_box',
      '#item'       => $item,
      '#delta'      => $delta,
      '#attributes' => $item_attributes,
      '#settings'   => $item_settings,
    ];

    $variables['items'][$delta] = $box;
    unset($box);
  }
}

/**
 * Prepares variables for gridstack-box.html.twig templates.
 */
function template_preprocess_gridstack_box(&$variables) {
  foreach (['attributes', 'delta', 'item', 'settings'] as $key) {
    $default = $key == 'delta' ? 0 : [];
    $variables[$key] = isset($variables['element']["#$key"]) ? $variables['element']["#$key"] : $default;
  }

  $item     = &$variables['item'];
  $settings = &$variables['settings'];

  // Boxes may have captions.
  foreach (['alt', 'category', 'data', 'link', 'title'] as $key) {
    $item['caption'][$key] = isset($item['caption']) && isset($item['caption'][$key]) ? $item['caption'][$key] : [];
  }

  $item['box']           = isset($item['box']) ? $item['box'] : [];
  $item['caption']       = array_filter($item['caption']);
  $settings['type']      = isset($settings['type']) ? $settings['type'] : '';
  $settings['use_stamp'] = $settings['type'] == 'stamp';

  $classes = empty($variables['attributes']['class']) ? [] : $variables['attributes']['class'];
  $variables['attributes']['class'] = array_merge(['gridstack__box', 'box'], $classes);
  $variables['attributes']['data-delta'] = $variables['delta'];
  $variables['wrapper'] = empty($settings['wrapper']) ? 'div' : $settings['wrapper'];
}
